import React, {Component} from 'react'
import PropTypes from 'prop-types'
import moment from 'moment'
import { Button, Affix, Modal, notification, Spin, message } from 'antd'
import * as XLSX from 'xlsx'
import MutilForm from '@/tabviews/tableshare/mutilform'
import ExcelIn from '../excelin'
import Utils from '@/utils/utils.js'
import Api from '@/api'
import './index.scss'

const { confirm } = Modal

class MainAction extends Component {
  static propTpyes = {
    BID: PropTypes.string,            // 主表ID
    BData: PropTypes.any,             // 主表数据
    Tab: PropTypes.any,               // 如果当前元素为标签时，tab为标签信息
    type: PropTypes.string,           // 判断当前为主表（main）、子表（sub）、子表标签（subtab）
    MenuID: PropTypes.string,         // 菜单ID
    actions: PropTypes.array,         // 按钮组
    logcolumns: PropTypes.array,      // 显示列
    dict: PropTypes.object,           // 字典项
    setting: PropTypes.any,           // 页面通用设置
    ContainerId: PropTypes.any,       // tab页面ID，用于弹窗控制
    refreshdata: PropTypes.func,      // 执行完成后数据刷新
    triggerPopview: PropTypes.func,   // 弹窗标签页触发
    getexceloutparam: PropTypes.func,  // 获取excel导出数据
    gettableselected: PropTypes.func  // 获取表格中数据
  }

  state = {
    visible: false,
    formdata: null,
    tabledata: null,
    confirmLoading: false,
    execAction: null,
    loadingUuid: '',
    btnloading: false,
    configMap: {}
  }

  refreshdata = (item, type) => {
    this.props.refreshdata(item, type)
  }
  
  /**
   * @description 触发按钮操作
   */
  actionTrigger = (item, record) => {
    const { setting, type, Tab, BID } = this.props

    if (type === 'sub' && Tab && Tab.supMenu && !BID) {
      notification.warning({
        top: 92,
        message: '需要上级主键值！',
        duration: 10
      })
      return
    }

    let _this = this
    let data = this.props.gettableselected() || []
    
    if (record) { // 表格中触发按钮
      data = [record]
    }

    if (item.Ot !== 'notRequired' && data.length === 0) {
      // 需要选择行时，校验数据
      notification.warning({
        top: 92,
        message: this.props.dict['main.action.confirm.selectline'],
        duration: 10
      })
      return
    } else if (item.Ot === 'requiredSgl' && data.length !== 1) {
      // 需要选择单行时，校验数据
      notification.warning({
        top: 92,
        message: this.props.dict['main.action.confirm.selectSingleLine'],
        duration: 10
      })
      return
    } else if (item.Ot !== 'notRequired' && !setting.primaryKey) {
      // 需要选择行时，校验是否设置主键
      notification.warning({
        top: 92,
        message: '未设置主键！',
        duration: 10
      })
      return
    }

    if (item.OpenType === 'prompt') {
      confirm({
        title: this.props.dict['main.action.confirm.tip'],
        onOk() {
          return new Promise(resolve => {
            _this.execSubmit(item, data, resolve)
          })
        },
        onCancel() {}
      })
    } else if (item.OpenType === 'exec') {
      this.setState({loadingUuid: item.uuid})
      this.execSubmit(item, data, () => {
        this.setState({loadingUuid: ''})
      })
    } else if (item.OpenType === 'pop') {
      this.setState({
        execAction: item,
        tabledata: data,
        btnloading: true
      }, () => {
        this.improveAction(item)
      })
    } else if (item.OpenType === 'excelOut') {
      if (
        item.intertype === 'inner' && !item.innerFunc &&
        ((setting.interType === 'inner' && setting.innerFunc) || setting.interType === 'outer')
      ) {
        notification.warning({
          top: 92,
          message: '导出按钮配置错误！',
          duration: 10
        })
        return
      }

      this.triggerExcelout(item)

    } else if (item.OpenType === 'excelIn') {
      if (item.verify && item.verify.sheet && item.verify.columns && item.verify.columns.length > 0) {
        let primaryId = '' // 导入时行Id
        if (item.Ot === 'requiredSgl') {
          primaryId = data[0][setting.primaryKey] || ''
        }

        this.refs.excelIn.exceltrigger(item, primaryId)
      } else {
        notification.warning({
          top: 92,
          message: 'excel导入验证信息未设置！',
          duration: 10
        })
      }
    } else if (item.OpenType === 'popview' && this.props.type !== 'subtab') {
      this.props.triggerPopview(item, data)
    } else if (item.OpenType === 'popview' && this.props.type === 'subtab') {
      notification.warning({
        top: 92,
        message: '弹窗页面不支持此设置！',
        duration: 10
      })
    } else if (item.OpenType === 'outerpage') {
      let url = item.url
      if (item.Ot === 'requiredSgl' && setting.primaryKey) {
        url = url + '?ID=' + data[0][setting.primaryKey]
      } else if (item.Ot !== 'notRequired' && !setting.primaryKey) {
        let ids = data.map(_data => _data[setting.primaryKey]).join(',')
        url = url + '?ID=' + ids
      }
      window.open(url)
    } else if (item.OpenType === 'tab' || item.OpenType === 'blank') {
      this.props.triggerPopview(item, data)
    } else {
      notification.warning({
        top: 92,
        message: '完善中。。。',
        duration: 10
      })
    }
  }

  /**
   * @description 按钮提交执行
   */
  execSubmit = (btn, data, _resolve, formdata) => {
    const { setting, logcolumns } = this.props

    if (btn.intertype === 'inner') {
      // 使用内部接口时，内部函数和数据源不可同时为空, 使用系统函数时，类型不可为空
      if (!btn.innerFunc && (!btn.sql || (btn.sql && !btn.sqlType))) {
        this.actionSettingError()
        _resolve()
        return
      }

      // 执行方式为多行拼接，且打开方式为表单时，会转为循环发送请求
      // 打开方式为模态框，使用内部函数添加(有批量添加场景，已去除)
      if (
        btn.Ot === 'notRequired' ||
        btn.Ot === 'requiredSgl' ||
        (btn.Ot === 'requiredOnce' && btn.OpenType !== 'pop')
        // (btn.OpenType === 'pop' && !btn.innerFunc && btn.sql && btn.sqlType === 'insert')
      ) {

        // 创建凭证时，需要选择行时
        if (data.length === 0 && !btn.innerFunc && btn.verify && btn.verify.voucher && btn.verify.voucher.enabled) {
          notification.warning({
            top: 92,
            message: '使用创建凭证函数，需要选择行！',
            duration: 10
          })
          return
        }

        let param = { // 系统存储过程
          func: 'sPC_TableData_InUpDe',
          BID: this.props.BID
        }
        let primaryId = setting.primaryKey && data[0] ? (data[0][setting.primaryKey] || '') : ''

        if (btn.OpenType === 'prompt' || btn.OpenType === 'exec') { // 是否弹框或直接执行
          let ID = ''
          if (btn.Ot === 'notRequired') {
            
          } else if (btn.Ot === 'requiredSgl') {
            ID = data[0][setting.primaryKey]
          } else if (btn.Ot === 'requiredOnce') { // id值拼接
            let ids = data.map(d => { return d[setting.primaryKey]})
            ID = ids.join(',')
          }

          if (btn.innerFunc) { // 使用自定义函数
            param.func = btn.innerFunc
            if (setting.primaryKey) { // 主键存在时，设置主键参数
              param[setting.primaryKey] = ID
            }
          } else if (btn.sql) {
            param.ID = primaryId
            param.LText = Utils.formatOptions(Utils.getSysDefaultSql(btn, setting, '', param, data[0], logcolumns)) // 数据源
            param.timestamp = moment().format('YYYY-MM-DD HH:mm:ss') + '.000'
            param.secretkey = Utils.encrypt(param.LText, param.timestamp)
          }
        } else if (btn.OpenType === 'pop') { // 表单
          if (btn.innerFunc) {
            param.func = btn.innerFunc

            if (setting.primaryKey) { // 主键存在时，设置主键参数
              param[setting.primaryKey] = primaryId
            }

            formdata.forEach(_data => {
              param[_data.key] = _data.value
            })

          } else if (btn.sql && btn.sqlType === 'insert') { // 系统函数添加时，生成uuid
            param.ID = Utils.getguid()
            param.LText = Utils.formatOptions(Utils.getSysDefaultSql(btn, setting, formdata, param, data[0], logcolumns)) // 数据源
            param.timestamp = moment().format('YYYY-MM-DD HH:mm:ss') + '.000'
            param.secretkey = Utils.encrypt(param.LText, param.timestamp)
          } else if (btn.sql) {
            param.ID = primaryId
            param.LText = Utils.formatOptions(Utils.getSysDefaultSql(btn, setting, formdata, param, data[0], logcolumns)) // 数据源
            param.timestamp = moment().format('YYYY-MM-DD HH:mm:ss') + '.000'
            param.secretkey = Utils.encrypt(param.LText, param.timestamp)
          }
        }

        Api.genericInterface(param).then((res) => {
          if (res.status) {
            this.execSuccess(btn, res)
          } else {
            this.execError(res, btn)
          }
          _resolve()
        })
      } else if (btn.Ot === 'required' || (btn.Ot === 'requiredOnce' && btn.OpenType === 'pop')) {
        let _params = data.map((cell, index) => {
          let param = {
            func: 'sPC_TableData_InUpDe',
            BID: this.props.BID
          }
          let primaryId = setting.primaryKey ? cell[setting.primaryKey] : ''

          if (btn.OpenType === 'prompt' || btn.OpenType === 'exec') { // 是否弹框或直接执行

            if (btn.innerFunc) {
              param.func = btn.innerFunc
              if (setting.primaryKey) {
                param[setting.primaryKey] = primaryId
              }
            } else if (btn.sql) {
              param.ID = primaryId
              param.LText = Utils.formatOptions(Utils.getSysDefaultSql(btn, setting, '', param, cell, logcolumns)) // 数据源
              param.timestamp = moment().format('YYYY-MM-DD HH:mm:ss') + '.000'
              param.secretkey = Utils.encrypt(param.LText, param.timestamp)
            }
          } else if (btn.OpenType === 'pop') { // 表单
            if (btn.innerFunc) {
              param.func = btn.innerFunc

              formdata.forEach(_data => {
                if (index !== 0 && _data.readin && cell.hasOwnProperty(_data.key)) {
                  param[_data.key] = cell[_data.key]
                } else {
                  param[_data.key] = _data.value
                }
              })

              if (setting.primaryKey) {
                param[setting.primaryKey] = primaryId
              }
            } else if (btn.sql) {
              if (index !== 0) {
                formdata = formdata.map(_data => {
                  if (_data.readin && cell.hasOwnProperty(_data.key)) {
                    _data.value = cell[_data.key]
                  }
                  return _data
                })
              }

              param.ID = primaryId
              param.LText = Utils.formatOptions(Utils.getSysDefaultSql(btn, setting, formdata, param, cell, logcolumns)) // 数据源
              param.timestamp = moment().format('YYYY-MM-DD HH:mm:ss') + '.000'
              param.secretkey = Utils.encrypt(param.LText, param.timestamp)
            }
          }

          return param
        })

        if (_params.length <= 20) {
          let deffers = _params.map(param => {
            return new Promise(resolve => {
              Api.genericInterface(param).then(res => {
                resolve(res)
              })
            })
          })
          Promise.all(deffers).then(result => {
            let iserror = false
            let errorMsg = ''
            result.forEach(res => {
              if (res.status) {
                errorMsg = res
              } else {
                iserror = true
                errorMsg = res
              }
            })
            if (!iserror) {
              this.execSuccess(btn, errorMsg)
            } else {
              this.execError(errorMsg, btn)
            }
            _resolve()
          })
        } else { // 超出20个请求时循环执行
          this.innerLoopRequest(_params, btn, _resolve)
        }
      } else {
        this.actionSettingError()
        _resolve()
        return
      }
    } else if (btn.intertype === 'outer') {
      /** *********************调用外部接口************************* */
      if (!btn.interface) { // 接口地址不存在时报错
        this.actionSettingError()
        _resolve()
        return
      }

      let _params = [] // 请求参数数组

      if (btn.Ot === 'notRequired' || btn.Ot === 'requiredSgl' || btn.Ot === 'requiredOnce') {
        let param = {
          BID: this.props.BID
        }

        if (btn.OpenType === 'pop' && formdata) { // 表单
          formdata.forEach(_data => {
            param[_data.key] = _data.value
          })
        }

        // 获取id
        if (btn.Ot === 'notRequired') {
          
        } else if (btn.Ot === 'requiredSgl' && setting.primaryKey) {
          param[setting.primaryKey] = data[0][setting.primaryKey]
        } else if (btn.Ot === 'requiredOnce' && setting.primaryKey) {
          let ids = data.map(d => { return d[setting.primaryKey]})
          param[setting.primaryKey] = ids.join(',')
        }

        _params.push(param)
      } else if (btn.Ot === 'required') {
        // 选择多行，循环调用
        _params = data.map((cell, index) => {
          let _cell = {
            BID: this.props.BID,
          }

          let _formparam = {}
          if (btn.OpenType === 'pop' && formdata) { // 表单
            formdata.forEach(_data => {
              if (index !== 0 && _data.readin && cell.hasOwnProperty(_data.key)) {
                _formparam[_data.key] = cell[_data.key]
              } else {
                _formparam[_data.key] = _data.value
              }
            })
          }

          if (setting.primaryKey) {
            _cell[setting.primaryKey] = cell[setting.primaryKey]
          }

          _cell = {..._formparam, ..._cell}

          return _cell
        })
      } else {
        this.actionSettingError()
        _resolve()
        return
      }

      // 循环调用外部接口（包括内部及回调函数）
      this.outerLoopRequest(_params, btn, _resolve)
      
    } else {
      this.actionSettingError()
      _resolve()
      return
    }
  }

  /**
   * @description 内部请求循环执行
   */
  innerLoopRequest = (params, btn, _resolve) => {
    if (!params && params.length === 0) return

    let param = params.shift()

    Api.genericInterface(param).then(res => {
      if (res.status) {
        if (params.length === 0) {
          this.execSuccess(btn, res)
          _resolve()
        } else {
          this.innerLoopRequest(params, btn, _resolve)
        }
      } else {
        this.execError(res, btn)
        _resolve()
      }
    })
  }

  /**
   * @description 外部请求循环执行
   */
  outerLoopRequest = (params, btn, _resolve) => {
    if (!params && params.length === 0) return

    let param = params.shift()
    let _outParam = null

    new Promise(resolve => {
      // 内部请求
      if (btn.innerFunc) {
        param.func = btn.innerFunc
        // 存在内部函数时，数据预处理
        Api.genericInterface(param).then(res => {
          if (res.status) {
            delete res.ErrCode
            delete res.ErrMesg
            delete res.message
            delete res.status

            // 使用处理后的数据调用外部接口
            resolve(res)
          } else {
            this.execError(res, btn)
            resolve(false)
            _resolve()
          }
        })
      } else {
        resolve(param)
      }
    }).then(res => {
      if (!res) return
      // 外部请求
      _outParam = JSON.parse(JSON.stringify(res))

      if (btn.sysInterface === 'true') {
        res.rduri = window.GLOB.mainSystemApi || window.GLOB.subSystemApi
      } else {
        res.rduri = btn.interface
      }
      // res.method = btn.method
      if (btn.outerFunc) {
        res.func = btn.outerFunc
      }

      res.appkey = window.GLOB.appkey || '' // 外部请求时，统一添加appkey

      return Api.genericInterface(res)
    }).then(response => {
      if (!response) return
      // 回调请求
      if (btn.callbackFunc) {
        // 存在回调函数时，调用
        delete response.message
        delete response.status

        response.func = btn.callbackFunc

        let _callbackparam = {..._outParam, ...response}
        return Api.genericInterface(_callbackparam)
      } else {
        if (response.status) {
          // 一次请求成功，进行下一项请求

          if (params.length === 0) {
            this.execSuccess(btn, response)
            _resolve()
          } else {
            this.outerLoopRequest(params, btn, _resolve)
          }
        } else {
          this.execError(response, btn)
          _resolve()
        }
      }
    }).then(res => {
      if (!res) return

      if (res.status) {
        if (params.length === 0) {
          this.execSuccess(btn, res)
          _resolve()
        } else {
          this.outerLoopRequest(params, btn, _resolve)
        }
      } else {
        this.execError(res, btn)
        _resolve()
      }
    })
  }

  /**
   * @description 操作成功后处理
   * 1、excel导出，成功后取消导出按钮加载中状态
   * 2、状态码为 S 时，显示成功信息后系统默认信息
   * 3、状态码为 -1 时，不显示任何信息
   * 4、模态框执行成功后是否关闭
   * 5、通知主列表刷新
   */
  execSuccess = (btn, res) => {
    if (res && res.ErrCode === 'S') { // 执行成功
      notification.success({
        top: 92,
        message: res.ErrMesg || this.props.dict['main.action.confirm.success'],
        duration: btn.verify && btn.verify.stime ? btn.verify.stime : 2
      })
    } else if (res && res.ErrCode === '-1') { // 完成后不提示

    } else if (res.Relogin === 'true') { // 切换用户功能
      sessionStorage.setItem('avatar', res.icon || '')
      sessionStorage.setItem('UserID', res.UserID)
      sessionStorage.setItem('SessionUid', Utils.getuuid())
      sessionStorage.setItem('LoginUID', res.LoginUID)
      sessionStorage.setItem('User_Name', res.UserName)
      
      window.location.reload()
      return
    }
    
    if (btn.OpenType === 'pop' && btn.setting && btn.setting.finish !== 'unclose') {
      this.setState({
        visible: false
      })
    } else if (btn.OpenType === 'excelOut' || btn.OpenType === 'excelIn') { // 导出excel
      this.setState({
        loadingUuid: ''
      })
    }

    this.refreshdata(btn, 'success')
  }

  /**
   * @description 操作失败后处理
   * 1、状态码为 E、N、F、NM 时，显示相应提示信息
   * 2、excel导出，失败后取消导出按钮加载中状态
   * 3、通知主列表刷新
   */
  execError = (res, btn) => {
    if (res.ErrCode === 'E') {
      Modal.error({
        title: res.message || res.ErrMesg,
      })
    } else if (res.ErrCode === 'N') {
      notification.error({
        top: 92,
        message: res.message || res.ErrMesg,
        duration: btn.verify && btn.verify.ntime ? btn.verify.ntime : 15
      })
    } else if (res.ErrCode === 'F') {
      notification.error({
        className: 'notification-custom-error',
        top: 92,
        message: res.message || res.ErrMesg,
        duration: btn.verify && btn.verify.ftime ? btn.verify.ftime : 15
      })
    } else if (res.ErrCode === 'NM') {
      message.error(res.message || res.ErrMesg)
    }
    
    if (btn.OpenType === 'excelOut' || btn.OpenType === 'excelIn') {
      this.setState({
        loadingUuid: ''
      })
    }
    
    this.refreshdata(btn, 'error')
  }

  /**
   * @description 按钮配置信息错误提示
   */
  actionSettingError = () => {
    notification.warning({
      top: 92,
      message: this.props.dict['main.action.settingerror'],
      duration: 10
    })
  }

  /**
   * @description 获取按钮配置信息
   */
  improveAction = (action) => {
    const { configMap, execAction } = this.state

    let _config = configMap[action.uuid]

    if (_config) {
      this.setState({
        execAction: {..._config, ...execAction}
      }, () => {
        this.improveActionForm()
      })
    } else {
      Api.getSystemCacheConfig({
        func: 'sPC_Get_LongParam',
        MenuID: action.uuid
      }).then(res => {
        let _LongParam = ''

        if (res.status && res.LongParam) {
          _LongParam = window.decodeURIComponent(window.atob(res.LongParam))
          try {
            _LongParam = JSON.parse(_LongParam)
          } catch (e) {
            _LongParam = ''
          }
        }

        if (!res.status) {
          notification.warning({
            top: 92,
            message: res.message,
            duration: 10
          })
          this.setState({
            execAction: null,
            tabledata: null,
            btnloading: false
          })
        } else if (!_LongParam || (action.OpenType === 'pop' && _LongParam.type !== 'Modal')) {
          notification.warning({
            top: 92,
            message: '未获取到按钮配置信息！',
            duration: 10
          })
          this.setState({
            execAction: null,
            tabledata: null,
            btnloading: false
          })
        } else {
          this.setState({
            configMap: {...configMap, [action.uuid]: _LongParam},
            execAction: {..._LongParam, ...execAction}
          }, () => {
            this.improveActionForm()
          })
        }
      })
    }
  }

  /**
   * @description 获取下拉表单选项信息
   */
  improveActionForm = () => {
    const { configMap, execAction } = this.state
    let subfields = []

    if (execAction.setting.display === 'prompt') { // 如果表单以是否框展示，不请求下拉菜单信息
      this.setState({
        btnloading: false
      })
      this.modelconfirm()
      return
    }

    if (execAction.groups.length > 0) {
      execAction.groups.forEach(group => {
        group.sublist.forEach(field => {
          if ((field.type === 'select' || field.type === 'link' || field.type === 'multiselect') && field.resourceType === '1' && field.dataSource) {
            subfields.push(field)
          }
        })
      })
    } else {
      execAction.fields.forEach(field => {
        if ((field.type === 'select' || field.type === 'link' || field.type === 'multiselect') && field.resourceType === '1' && field.dataSource) {
          subfields.push(field)
        }
      })
    }

    if (subfields.length === 0) {
      this.setState({
        visible: true,
        btnloading: false
      })
      return
    }

    let deffers = subfields.map(item => {
      let _option = Utils.getSelectQueryOptions(item)
      let _sql = Utils.formatOptions(_option.sql)
      let isSSO = item.database === 'sso'

      let param = {
        func: 'sPC_Get_SelectedList',
        LText: _sql,
        obj_name: 'data',
        arr_field: _option.field
      }

      param.timestamp = moment().format('YYYY-MM-DD HH:mm:ss') + '.000'
      param.secretkey = Utils.encrypt(param.LText, param.timestamp)

      return new Promise(resolve => {
        Api.getSystemCacheConfig(param, isSSO).then(res => {
          res.search = item
          resolve(res)
        })
      })
    })

    let _field = {}
    let error = ''
    Promise.all(deffers).then(result => {
      result.forEach(res => {
        if (res.status) {
          let options = res.data.map(cell => {
            let item = {
              key: Utils.getuuid(),
              Value: cell[res.search.valueField],
              Text: cell[res.search.valueText]
            }

            if (res.search.type === 'link') {
              item.parentId = cell[res.search.linkField]
            } else if (res.search.type === 'select' && res.search.linkSubField && res.search.linkSubField.length > 0) {
              res.search.linkSubField.forEach(_field => {
                item[_field] = (cell[_field] || cell[_field] === 0) ? cell[_field] : ''
              })
            }

            return item
          })

          _field[res.search.uuid] = options
        } else {
          error = res
        }
      })

      if (error) {
        notification.warning({
          top: 92,
          message: error.message,
          duration: 10
        })
      }

      this.setState({
        configMap: {...configMap, ..._field}
      }, () => {
        this.setState({
          visible: true,
          btnloading: false
        })
      })
    })
  }

  /**
   * @description Excel 导入
   */
  getexceldata = (data, btn, errors, primaryId) => {
    if (errors) {
      if (errors === 'notexit') {
        notification.warning({
          top: 92,
          message: '工作表《' + btn.verify.sheet + '》不存在！',
          duration: 10
        })
      } else if (errors === 'empty') {
        notification.warning({
          top: 92,
          message: '工作表《' + btn.verify.sheet + '》为空！',
          duration: 10
        })
      } else if (errors === 'headerError') {
        notification.warning({
          top: 92,
          message: '工作表《' + btn.verify.sheet + '》表头设置错误！',
          duration: 10
        })
      }
      
      return
    }

    if (!data || data.length === 0) {
      notification.warning({
        top: 92,
        message: '未获取到工作表《' + btn.verify.sheet + '》数据！',
        duration: 10
      })
      return
    }

    let result = Utils.getExcelInSql(btn, data, this.props.dict)
    if (result.errors) {
      notification.warning({
        top: 92,
        message: result.errors,
        duration: 10
      })
      return
    }

    let param = {
      BID: this.props.BID,
      ID: primaryId
    }

    param.LText = Utils.formatOptions(result.sql)
    param.timestamp = moment().format('YYYY-MM-DD HH:mm:ss') + '.000'
    param.secretkey = Utils.encrypt(param.LText, param.timestamp)

    this.setState({loadingUuid: btn.uuid})

    if (btn.intertype === 'inner' && !btn.innerFunc) { // 系统存储过程
      param.func = 'sPC_TableData_InUpDe'

      Api.genericInterface(param).then((res) => {
        if (res.status) {
          this.execSuccess(btn, res)
        } else {
          this.execError(res, btn)
        }
      })
    } else if (btn.intertype === 'inner' && btn.innerFunc) { // 自定义存储过程
      param.func = btn.innerFunc

      Api.genericInterface(param).then((res) => {
        if (res.status) {
          this.execSuccess(btn, res)
        } else {
          this.execError(res, btn)
        }
      })
    } else if (btn.intertype === 'outer') { // 外部接口
      let _outParam = null

      new Promise(resolve => {
        // 内部请求
        if (btn.innerFunc) {
          param.func = btn.innerFunc
          // 存在内部函数时，数据预处理
          Api.genericInterface(param).then(res => {
            if (res.status) {
              delete res.ErrCode
              delete res.ErrMesg
              delete res.message
              delete res.status

              // 使用处理后的数据调用外部接口
              resolve(res)
            } else {
              this.execError(res, btn)
              resolve(false)
            }
          })
        } else {
          resolve(param)
        }
      }).then(res => {
        if (!res) return
        // 外部请求
        _outParam = JSON.parse(JSON.stringify(res))

        if (btn.sysInterface === 'true') {
          res.rduri = window.GLOB.mainSystemApi || window.GLOB.subSystemApi
        } else {
          res.rduri = btn.interface
        }

        if (btn.outerFunc) {
          res.func = btn.outerFunc
        }

        res.appkey = window.GLOB.appkey || '' // 外部请求时，统一添加appkey

        return Api.genericInterface(res)
      }).then(response => {
        if (!response) return
        // 回调请求
        if (btn.callbackFunc) {
          // 存在回调函数时，调用
          delete response.message
          delete response.status

          response.func = btn.callbackFunc

          let _callbackparam = {..._outParam, ...response}
          return Api.genericInterface(_callbackparam)
        } else {
          if (response.status) {
            this.execSuccess(btn, response)
          } else {
            this.execError(response, btn)
          }
        }
      }).then(res => {
        if (!res) return

        if (res.status) {
          this.execSuccess(btn, res)
        } else {
          this.execError(res, btn)
        }
      })
    }
  }

  /**
   * @description Excel 导出
   */
  triggerExcelout = (btn) => {
    let viewParam = this.props.getexceloutparam()
    let name = `${viewParam.menuName}${moment().format('YYYYMMDDHHmmss')}.xlsx`
    let pageSize = 1000
    
    this.setState({loadingUuid: btn.uuid})

    if (btn.pagination !== 'true') {
      if (btn.intertype === 'inner' && !btn.innerFunc) { // 使用系统函数
        if (!viewParam.arr_field) {
          this.execError({ErrCode: 'N', message: '未设置显示列！'}, btn)
          return
        }

        let param = this.getExcelDefaultParam(viewParam.arr_field, viewParam.orderBy, viewParam.search)

        Api.genericInterface(param).then(result => {
          if (result.status) {
            this.exportExcel(result.data, btn, name)
          } else {
            this.execError(result, btn)
          }
        })
      } else if (btn.intertype === 'inner' && btn.innerFunc) { // 使用内部函数
        let param = this.getExcelCustomParam(viewParam.orderBy, viewParam.search)
        param.func = btn.innerFunc

        Api.genericInterface(param).then(result => {
          if (result.status) {
            this.exportExcel(result.data, btn, name)
          } else {
            this.execError(result, btn)
          }
        })
      } else if (btn.intertype === 'outer' && !btn.innerFunc) { // 使用外部函数
        let param = this.getExcelCustomParam(viewParam.orderBy, viewParam.search)
        if (btn.sysInterface === 'true') {
          param.rduri = window.GLOB.mainSystemApi || window.GLOB.subSystemApi
        } else {
          param.rduri = btn.interface
        }
  
        param.appkey = window.GLOB.appkey || ''
  
        if (btn.outerFunc) {
          param.func = btn.outerFunc
        }

        Api.genericInterface(param).then(result => {
          if (result.status) {
            this.exportExcel(result.data, btn, name)
          } else {
            this.execError(result, btn)
          }
        })
      } else if (btn.intertype === 'outer' && btn.innerFunc) {
        let param = this.getExcelCustomParam(viewParam.orderBy, viewParam.search)
        param.func = btn.innerFunc

        Api.genericInterface(param).then(res => {
          if (res.status) {
            delete res.ErrCode
            delete res.ErrMesg
            delete res.message
            delete res.status

            if (btn.sysInterface === 'true') {
              res.rduri = window.GLOB.mainSystemApi || window.GLOB.subSystemApi
            } else {
              res.rduri = btn.interface
            }
  
            if (btn.outerFunc) {
              res.func = btn.outerFunc
            }
      
            res.appkey = window.GLOB.appkey || '' // 外部请求时，统一添加appkey
      
            Api.genericInterface(res).then(result => {
              if (result.status) {
                this.exportExcel(result.data, btn, name)
              } else {
                this.execError(result, btn)
              }
            })
          } else {
            this.execError(res, btn)
          }
        })
      } else {
        this.execError({ErrCode: 'N', message: '导出按钮设置错误！'}, btn)
      }
    } else if (btn.intertype === 'outer' && btn.innerFunc) { // 分页，且两步请求
      this.getExcelOutDoubleData(btn, viewParam, 1, pageSize, [], name)
    } else { // 分页，一步请求
      this.getExcelOutData(btn, viewParam, 1, pageSize, [], name)
    }
  }

  /**
   * @description 两步分页请求
   */
  getExcelOutDoubleData = (btn, viewParam, pageIndex, pageSize, data, name) => {
    let param = this.getExcelCustomParam(viewParam.orderBy, viewParam.search, true, pageIndex, pageSize)
    param.func = btn.innerFunc

    Api.genericInterface(param).then(res => {
      if (res.status) {
        delete res.ErrCode
        delete res.ErrMesg
        delete res.message
        delete res.status

        if (btn.sysInterface === 'true') {
          res.rduri = window.GLOB.mainSystemApi || window.GLOB.subSystemApi
        } else {
          res.rduri = btn.interface
        }

        if (btn.outerFunc) {
          res.func = btn.outerFunc
        }
  
        res.appkey = window.GLOB.appkey || '' // 外部请求时，统一添加appkey
  
        Api.genericInterface(res).then(result => {
          if (result.status) {
            if (!result.data) {
              this.execError({ErrCode: 'N', message: '未获取到数据信息！'}, btn)
            } else if (result.data.length >= pageSize) {
              data = data.concat(result.data)
              pageIndex++
              this.getExcelOutDoubleData(btn, viewParam, pageIndex, pageSize, data, name)
            } else {
              data = data.concat(result.data)
              this.exportExcel(data, btn, name)
            }
          } else {
            this.execError(result, btn)
          }
        })
      } else {
        this.execError(res, btn)
      }
    })
  }

  /**
   * @description 一步分页请求
   */
  getExcelOutData = (btn, viewParam, pageIndex, pageSize, data, name) => {
    let param = null
    if (btn.intertype === 'inner' && !btn.innerFunc) { // 使用系统函数
      if (!viewParam.arr_field) {
        this.execError({ErrCode: 'N', message: '未设置显示列！'}, btn)
        return
      }

      param = this.getExcelDefaultParam(viewParam.arr_field, viewParam.orderBy, viewParam.search, true, pageIndex, pageSize)

      
    } else if (btn.intertype === 'inner' && btn.innerFunc) { // 使用内部函数
      param = this.getExcelCustomParam(viewParam.orderBy, viewParam.search, true, pageIndex, pageSize)
      param.func = btn.innerFunc

    } else if (btn.intertype === 'outer' && !btn.innerFunc) { // 使用外部函数
      param = this.getExcelCustomParam(viewParam.orderBy, viewParam.search, true, pageIndex, pageSize)
      if (btn.sysInterface === 'true') {
        param.rduri = window.GLOB.mainSystemApi || window.GLOB.subSystemApi
      } else {
        param.rduri = btn.interface
      }

      param.appkey = window.GLOB.appkey || ''

      if (btn.outerFunc) {
        param.func = btn.outerFunc
      }
    }

    Api.genericInterface(param).then(result => {
      if (result.status) {
        if (!result.data) {
          this.execError({ErrCode: 'N', message: '未获取到数据信息！'}, btn)
        } else if (result.data.length >= pageSize) {
          data = data.concat(result.data)
          pageIndex++
          this.getExcelOutData(btn, viewParam, pageIndex, pageSize, data, name)
        } else {
          data = data.concat(result.data)
          this.exportExcel(data, btn, name)
        }
      } else {
        this.execError(result, btn)
      }
    })
  }

  /**
   * @description Excel 生成
   */
  exportExcel = (data, btn, name) => {
    const { logcolumns } = this.props
    if (data && data.length > 0) {
      try {
        let _header = []
        let _topRow = {}
        let colwidth = []

        let hidecolumns = []
        logcolumns.forEach(col => {
          if (col.Hide === 'true') {
            hidecolumns.push(col.field)
          }
        })

        if (btn.verify && btn.verify.columns && btn.verify.columns.length > 0) {
          btn.verify.columns.forEach(col => {
            if (hidecolumns.includes(col.Column)) return
            if (!data[0].hasOwnProperty(col.Column)) return
            if (_topRow[col.Column]) return

            _header.push(col.Column)
            _topRow[col.Column] = col.Text

            colwidth.push({width: col.Width})
          })
        }

        logcolumns.forEach(col => {
          if (col.Hide === 'true') return
          if (!data[0].hasOwnProperty(col.field)) return
          if (_topRow[col.field]) return

          _header.push(col.field)
          _topRow[col.field] = col.label

          let _colwidth = Math.floor(col.Width / 6)

          if (!_colwidth || _colwidth < 5) {
            _colwidth = 5
          }

          colwidth.push({width: _colwidth})
        })

        Object.keys(data[0]).forEach(key => {
          if (hidecolumns.includes(key)) return
          if (_topRow[key]) return

          _header.push(key)
          _topRow[key] = key

          colwidth.push({width: 12})
        })

        let table = []

        table.push(_topRow)

        data.forEach(item => {
          let _row = {}
          _header.forEach(field => {
            _row[field] = item[field]
          })

          table.push(_row)
        })

        const ws = XLSX.utils.json_to_sheet(table, {header: _header, skipHeader: true})

        ws['!cols'] = colwidth

        const wb = XLSX.utils.book_new()
        XLSX.utils.book_append_sheet(wb, ws, 'Sheet1')

        XLSX.writeFile(wb, name)

        this.execSuccess(btn, {ErrCode: 'S', ErrMesg: '导出成功！'})
      } catch {
        this.execError({ErrCode: 'N', message: 'Excel生成失败！'}, btn)
      }
      
    } else {
      this.execError({ErrCode: 'N', message: '未查询到导出数据！'}, btn)
    }
  }

  /**
   * @description 获取用户自定义存储过程传参
   */
  getExcelCustomParam = (orderBy, search, pagination = false, pageIndex = 1, pageSize = 100) => {
    let _search = Utils.formatCustomMainSearch(search)

    let param = {
      OrderCol: orderBy,
      ..._search
    }

    if (this.props.BID) {
      param.BID = this.props.BID
    }

    if (pagination) {
      param.PageIndex = pageIndex
      param.PageSize = pageSize
    }

    return param
  }

  /**
   * @description 获取默认存储过程请求参数
   */
  getExcelDefaultParam = (arr_field, orderBy, search, pagination = false, pageIndex = 1, pageSize = 100) => {
    const { setting } = this.props

    let _search = Utils.joinMainSearchkey(search)

    _search = _search ? 'where ' + _search : ''
    
    let param = {
      func: 'sPC_Get_TableData',
      obj_name: 'data',
      arr_field: arr_field,
      appkey: window.GLOB.appkey || ''
    }

    if (this.props.BID) {
      param.BID = this.props.BID
    }
    
    let _dataresource = setting.dataresource

    if (/\s/.test(_dataresource)) {
      _dataresource = '(' + _dataresource + ') tb'
    }

    if (setting.queryType === 'statistics') { // 统计数据源，内容替换
      let fieldmap = new Map()
      let options = search.map(item => {
        let _field = item.key

        if (fieldmap.has(_field)) {
          _field = _field + '1'
        }

        fieldmap.set(item.key, true)

        return {
          reg: new RegExp('@' + _field, 'ig'),
          value: item.value
        }
      })

      options.reverse()

      options.forEach(item => {
        _dataresource = _dataresource.replace(item.reg, `'${item.value}'`)
      })

      _search = ''
    }

    let LText = ''
    if (pagination) {
      LText = `select top ${pageSize} ${arr_field} from (select ${arr_field} ,ROW_NUMBER() over(order by ${orderBy}) as rows from ${_dataresource} ${_search}) tmptable where rows > ${pageSize * (pageIndex - 1)} order by tmptable.rows`
    } else {
      LText = `select ${arr_field} from (select ${arr_field} ,ROW_NUMBER() over(order by ${orderBy}) as rows from ${_dataresource} ${_search}) tmptable order by tmptable.rows`
    }

    param.LText = Utils.formatOptions(LText)
    param.timestamp = moment().format('YYYY-MM-DD HH:mm:ss') + '.000'
    param.secretkey = Utils.encrypt(param.LText, param.timestamp)
    param.DateCount = ''

    return param
  }

  /**
   * @description 模态框（表单），确认
   */
  handleOk = () => {
    this.formRef.handleConfirm().then(res => {
      this.setState({
        confirmLoading: true
      })

      this.execSubmit(this.state.execAction, this.state.tabledata, () => {
        this.setState({
          confirmLoading: false
        })
      }, res)
    }, () => {})
  }

  /**
   * @description 模态框（表单），取消
   */
  handleCancel = () => {
    this.setState({
      visible: false
    })
  }

  modelconfirm = () => {
    const { BData } = this.props
    const { execAction, tabledata } = this.state
    let _this = this
    let _fields = []
    
    if (execAction.groups.length > 0) {
      execAction.groups.forEach(group => {
        _fields = [..._fields, ...group.sublist]
      })
    } else {
      _fields = execAction.fields
    }

    let result = _fields.map(item => {
      let _readin = item.readin !== 'false'
      let _initval = item.initval

      if (item.type === 'linkMain' || item.type === 'funcvar') {
        _readin = false
      }

      if (item.type === 'linkMain' && BData && BData.hasOwnProperty(item.field)) {
        _initval = BData[item.field]
      } else if (_readin && tabledata[0] && tabledata[0].hasOwnProperty(item.field)) {
        _initval = tabledata[0][item.field]
      }

      let _fieldlen = item.fieldlength || 50
      if (item.type === 'textarea' || item.type === 'fileupload') {
        _fieldlen = item.fieldlength || 512
      } else if (item.type === 'number') {
        _fieldlen = item.decimal ? item.decimal : 0
      }

      return {
        key: item.field,
        readonly: item.readonly === 'true',
        readin: _readin,
        fieldlen: _fieldlen,
        type: item.type,
        value: _initval
      }
    })

    confirm({
      title: this.props.dict['main.action.confirm.tip'],
      onOk() {
        return new Promise(resolve => {
          _this.execSubmit(execAction, tabledata, resolve, result)
        })
      },
      onCancel() {}
    })
  }

  /**
   * @description 显示模态框
   */
  getModels = () => {
    const { execAction } = this.state

    if (!execAction || !this.state.visible) return

    let title = ''
    let width = '62vw'
    let clickouter = false
    let container = document.body

    if (execAction && execAction.setting) {
      title = execAction.setting.title
      width = execAction.setting.width + 'vw'

      if (execAction.setting.container === 'tab' && this.props.ContainerId) {
        width = execAction.setting.width + '%'
        container = () => document.getElementById(this.props.ContainerId)
      }

      if (execAction.setting.clickouter === 'close') {
        clickouter = true
      }
    }

    if (this.props.type === 'subtab') {
      container = document.body
    }

    return (
      <Modal
        title={title}
        maskClosable={clickouter}
        getContainer={container}
        wrapClassName='action-modal'
        visible={this.state.visible}
        width={width}
        onOk={this.handleOk}
        confirmLoading={this.state.confirmLoading}
        onCancel={this.handleCancel}
        destroyOnClose
      >
        <MutilForm
          dict={this.props.dict}
          action={execAction}
          inputSubmit={this.handleOk}
          configMap={this.state.configMap}
          data={this.state.tabledata[0]}
          BData={this.props.BData}
          wrappedComponentRef={(inst) => this.formRef = inst}
        />
      </Modal>
    )
  }

  render() {
    const { loadingUuid, btnloading } = this.state

    if (this.props.setting.actionfixed && this.props.type === 'main') { // 按钮是否固定在头部
      return (
        <Affix offsetTop={48}>
          <div className="button-list toolbar-button" id={this.props.MenuID + 'mainaction'}>
            {this.props.actions.map((item, index) => {
              if (loadingUuid === item.uuid) {
                return (
                  <Button
                    className={'mk-btn mk-' + item.class}
                    icon={item.icon}
                    key={'action' + index}
                    onClick={() => {this.actionTrigger(item)}}
                    loading
                  >{item.label}</Button>
                )
              } else {
                return (
                  <Button
                    className={'mk-btn mk-' + item.class}
                    icon={item.icon}
                    key={'action' + index}
                    onClick={() => {this.actionTrigger(item)}}
                  >{item.label}</Button>
                )
              }
            })}
            {this.getModels()}
            {btnloading && <Spin size="large" />}
            <ExcelIn MenuID={this.props.MenuID} returndata={this.getexceldata} ref="excelIn" />
          </div>
        </Affix>
      )
    } else {
      return (
        <div className="button-list toolbar-button">
          {this.props.actions.map((item, index) => {
            if (loadingUuid === item.uuid) {
              return (
                <Button
                  className={'mk-btn mk-' + item.class}
                  icon={item.icon}
                  key={'action' + index}
                  onClick={() => {this.actionTrigger(item)}}
                  loading
                >{item.label}</Button>
              )
            } else {
              return (
                <Button
                  className={'mk-btn mk-' + item.class}
                  icon={item.icon}
                  key={'action' + index}
                  onClick={() => {this.actionTrigger(item)}}
                >{item.label}</Button>
              )
            }
          })}
          {this.getModels()}
          {btnloading && <Spin size="large" />}
          <ExcelIn MenuID={this.props.MenuID} returndata={this.getexceldata} ref="excelIn" />
        </div>
      )
    }
  }
}

export default MainAction